@@ -1,8 +1,9 @@
+;;; -*- coding: iso-2022-7bit; -*-
 ;;; tramp-uu.el --- uuencode in Lisp
 
 ;; Copyright (C) 2002  Free Software Foundation, Inc.
 
-;; Author: Kai Grojohann <Kai.Grossjohann@CS.Uni-Dortmund.DE>
+;; Author: Kai Gro,A_(Bjohann <Kai.Grossjohann@CS.Uni-Dortmund.DE>
 ;; Keywords: comm, terminals
 
 ;; This file is free software; you can redistribute it and/or modify
@@ -22,7 +23,8 @@
 
 ;;; Commentary:
 
-;; 
+;; An implementation of "uuencode" in Lisp.  Uses the function
+;; base64-encode-region which is built-in to modern Emacsen.
 
 ;;; Code:
 
@@ -30,18 +32,27 @@
   "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/"
   "Mapping from base64-encoded character to the byte it represents.")
 
+(defvar tramp-uu-b64-char-to-byte
+  (let ((i 0))
+    (mapcar (lambda (c)
+	      (prog1
+		  (cons c i)
+		(setq i (1+ i))))
+	    tramp-uu-b64-alphabet))
+  "Alist of mapping from base64 character to its byte.")
+
 (defun tramp-uu-byte-to-uu-char (byte)
   "Return the character encoding BYTE."
-  (+ byte 32))
+  (if (zerop byte) ?` (+ byte 32)))
 
 (defun tramp-uu-b64-char-to-byte (char)
   "Return the byte that is encoded as CHAR."
-  (or
-   (string-match (regexp-quote (char-to-string char)) tramp-uu-b64-alphabet)
-   (error "Unknown base64 character `%c'" char)))
+  (cdr (assq char tramp-uu-b64-char-to-byte)))
 
 (defun tramp-uuencode-region (beg end)
   "UU-encode the region between BEG and END."
+  ;; First we base64 encode the region, then we transmogrify that into
+  ;; uu encoding.
   (let ((len (base64-encode-region beg end t))
 	(padding 0)
 	i c)
@@ -52,24 +63,30 @@
 	(setq c (char-after (point)))
 	(delete-char 1)
 	(if (equal c ?=)
-	    (progn
-	      (incf padding)
-	      (insert "`"))
+	    ;; "=" means padding.  Insert "`" instead.
+	    (insert "`")
 	  (insert (tramp-uu-byte-to-uu-char (tramp-uu-b64-char-to-byte c))))
-	(incf i)
-	(when (zerop (% i 45))
+	(setq i (1+ i))
+	;; Every 60 characters, add "M" at beginning of line (as
+	;; length byte) and insert a newline.
+	(when (zerop (% i 60))
 	  (save-excursion
 	    (beginning-of-line)
-	    (insert (char-to-string (+ 32 45))))
-	  (newline)))
-      (unless (zerop (% i 45))
+	    (insert (char-to-string (+ 32 (/ (* 3 60) 4)))))
+	  (insert "\n")))
+      ;; If there is something leftover, we compute the length byte
+      ;; for that stuff and insert it and a trailing newline.
+      (unless (zerop (% i 60))
 	(save-excursion
 	  (beginning-of-line)
-	  (insert (char-to-string (+ 32 (% i 45) (- padding))))
-	  (newline)))
+	  (insert (char-to-string (+ 32 (% (- end beg) 45)))))
+	(insert "\n"))
+      ;; Why is there always a "`" line at the end?
       (insert "`\nend\n")
       (goto-char beg)
       (insert "begin 600 xxx\n"))))
 
 (provide 'tramp-uu)
+
+;;; arch-tag: 7153f2c6-8be5-4cd2-8c06-0fbcf5190ef6
 ;;; tramp-uu.el ends here
